import { NativeModules } from 'react-native';
const funsdk = NativeModules.FunSDKDevAlarmModule;
// Максимум за 1 день?
/**
 * Searches alarm messages based on the provided parameters.
 *
 * @param {SearchAlarmMsgParams} params - The parameters for searching alarm messages.
 * @return {Promise<AlarmInfo[]>} A promise that resolves to an array of alarm information.
 */
export function searchAlarmMsg(params) {
  return funsdk.searchAlarmMsg(params);
}
/**
 * Deletes alarm information.
 *
 * @param {DeleteAlarmInfoParams} params - The parameters for deleting alarm information.
 * @param {string} params.deviceId - The ID of the device.
 * @param {'MSG' | 'VIDEO'} params.deleteType - The type of deletion ('MSG' or 'VIDEO').
 * @param {Array<{id: string}>} params.alarmInfos - The array of alarm information IDs to delete.
 * @throws {Error} Throws an error if the length of alarmInfos is more than 60.
 * @return {Promise<DeleteAlarmInfoResponse>} A promise that resolves to the response containing the success status and the deleted alarm information.
 */
export function deleteAlarmInfo(params) {
  if (params.alarmInfos.length > 60) {
    throw new Error("alarmInfos length can't be more than 60");
  }
  return funsdk.deleteAlarmInfo(params);
}

/**
 * Deletes all alarm messages based on the provided parameters, excluding the alarmInfos field.
 *
 * @param {Omit<DeleteAlarmInfoParams, 'alarmInfos'>} params - The parameters for deleting alarm information, excluding the alarmInfos field.
 * @return {Promise<Omit<DeleteAlarmInfoResponse, 'alarmInfos'>>} A promise that resolves to the response containing the success status and the deleted alarm information, excluding the alarmInfos field.
 */
export function deleteAllAlarmMsg(params) {
  return funsdk.deleteAllAlarmMsg(params);
}
//# sourceMappingURL=index.js.map