import type { AlarmType } from '../types/alarm';
export type SearchAlarmMsgParams = {
    deviceId: string;
    deviceChannel: number;
    alarmType: number;
    searchTime: number;
    searchDays: number;
    imgSizes: {
        imgHeight: number;
        imgWidth: number;
    };
};
export type AlarmInfo = {
    alarmRing: string | null;
    channel: number;
    devName: string | null;
    event: AlarmType | null;
    extInfo: string | null;
    id: string;
    isHavePic: boolean;
    linkCenter: {
        msgType: string | null;
        msg: string | null;
        subSn: string | null;
        modelType: number | null;
    } | null;
    pic: string | null;
    picError: number;
    picsize: number;
    pushMsg: string | null;
    serialNumber: string | null;
    startTime: string | null;
    status: string | null;
    videoInfo: boolean;
};
/**
 * Searches alarm messages based on the provided parameters.
 *
 * @param {SearchAlarmMsgParams} params - The parameters for searching alarm messages.
 * @return {Promise<AlarmInfo[]>} A promise that resolves to an array of alarm information.
 */
export declare function searchAlarmMsg(params: SearchAlarmMsgParams): Promise<AlarmInfo[]>;
export type DeleteAlarmInfoParams = {
    deviceId: string;
    deleteType: 'MSG' | 'VIDEO';
    alarmInfos: {
        id: string;
    }[];
};
export type DeleteAlarmInfoResponse = {
    isSuccess: boolean;
    alarmInfos: AlarmInfo[];
};
/**
 * Deletes alarm information.
 *
 * @param {DeleteAlarmInfoParams} params - The parameters for deleting alarm information.
 * @param {string} params.deviceId - The ID of the device.
 * @param {'MSG' | 'VIDEO'} params.deleteType - The type of deletion ('MSG' or 'VIDEO').
 * @param {Array<{id: string}>} params.alarmInfos - The array of alarm information IDs to delete.
 * @throws {Error} Throws an error if the length of alarmInfos is more than 60.
 * @return {Promise<DeleteAlarmInfoResponse>} A promise that resolves to the response containing the success status and the deleted alarm information.
 */
export declare function deleteAlarmInfo(params: DeleteAlarmInfoParams): Promise<DeleteAlarmInfoResponse>;
/**
 * Deletes all alarm messages based on the provided parameters, excluding the alarmInfos field.
 *
 * @param {Omit<DeleteAlarmInfoParams, 'alarmInfos'>} params - The parameters for deleting alarm information, excluding the alarmInfos field.
 * @return {Promise<Omit<DeleteAlarmInfoResponse, 'alarmInfos'>>} A promise that resolves to the response containing the success status and the deleted alarm information, excluding the alarmInfos field.
 */
export declare function deleteAllAlarmMsg(params: Omit<DeleteAlarmInfoParams, 'alarmInfos'>): Promise<Omit<DeleteAlarmInfoResponse, 'alarmInfos'>>;
//# sourceMappingURL=index.d.ts.map