import React from 'react';
import type { ViewProps } from 'react-native';
import { FUNSDK_MEDIA_PLAY_STATE_ENUM, STREAM_TYPE } from '../types/stream';
export type MonitorViewNativeProps = ViewProps & {
    devId: string;
    channelId: number;
    streamType: STREAM_TYPE;
    onMediaPlayState: (event: {
        nativeEvent: {
            state: FUNSDK_MEDIA_PLAY_STATE_ENUM | number;
        };
    }) => void;
    onShowRateAndTime: (event: {
        nativeEvent: {
            time: string;
            rate: string;
        };
    }) => void;
    onVideoBufferEnd: (event: {
        nativeEvent: {
            isBufferEnd: boolean;
        };
    }) => void;
    onGetInfo: (event: {
        nativeEvent: {
            type: 'streamType';
            streamType: number;
        };
    }) => void;
    onFailed: (event: {
        nativeEvent: {
            msgId: number;
            errorId: number;
        };
    }) => void;
    onStartInit: (event: {
        nativeEvent: {};
    }) => void;
    onDebugState: (event: {
        nativeEvent: {
            state: string;
        };
    }) => void;
    onCapture: (event: {
        nativeEvent: {
            path: string;
        };
    }) => void;
};
export declare const MonitorView: import("react-native").HostComponent<MonitorViewNativeProps>;
export type MonitorProps = ViewProps & {
    devId: string;
    channelId: number;
    streamType?: STREAM_TYPE;
    onStartInit?: () => void;
    onMediaPlayState?: (obj: {
        state: FUNSDK_MEDIA_PLAY_STATE_ENUM | number;
    }) => void;
    onShowRateAndTime?: (obj: {
        time: string;
        rate: string;
    }) => void;
    onVideoBufferEnd?: (obj: {
        isBufferEnd: boolean;
    }) => void;
    onGetInfo?: (obj: {
        type: 'streamType';
        streamType: number;
    }) => void;
    onFailed?: (obj: {
        msgId: number;
        errorId: number;
    }) => void;
    onDebugState?: (obj: {
        state: string;
    }) => void;
    onCapture?: (obj: {
        path: string;
    }) => void;
};
export declare class Monitor extends React.Component<MonitorProps, any> {
    private myRef;
    constructor(props: MonitorProps);
    playVideo(): void;
    pauseVideo(): void;
    replayVideo(): void;
    stopVideo(): void;
    destroyVideo(): void;
    getStreamType(): void;
    captureImage(path: string): void;
    startVideoRecord(path: string): void;
    stopVideoRecord(): void;
    openVoice(): void;
    closeVoice(): void;
    /**
     * set voice changer
     * 0 normal, 1 male ,2 female
     */
    startSingleIntercomAndSpeak(): void;
    stopSingleIntercomAndSpeak(): void;
    startDoubleIntercom(): void;
    stopDoubleIntercom(): void;
    switchStreamTypeMonitor(): void;
    updateStreamTypeMonitor(streamType: STREAM_TYPE): void;
    setVideoFullScreen(isFullScreen: boolean): void;
    capturePicFromDevAndSave(): void;
    changeVideoRatio(): void;
    _onStartInit: () => void;
    _onMediaPlayState: (event: Parameters<MonitorViewNativeProps['onMediaPlayState']>[0]) => void;
    _onShowRateAndTime: (event: Parameters<MonitorViewNativeProps['onShowRateAndTime']>[0]) => void;
    _onVideoBufferEnd: (event: Parameters<MonitorViewNativeProps['onVideoBufferEnd']>[0]) => void;
    _onGetInfo: (event: Parameters<MonitorViewNativeProps['onGetInfo']>[0]) => void;
    _onFailed: (event: {
        nativeEvent: {
            msgId: number;
            errorId: number;
        };
    }) => void;
    _onDebugState: (event: {
        nativeEvent: {
            state: string;
        };
    }) => void;
    _onCapture: (event: {
        nativeEvent: {
            path: string;
        };
    }) => void;
    render(): React.JSX.Element;
}
//# sourceMappingURL=index.d.ts.map