import React from 'react';
import { ViewProps } from 'react-native';
import type { FUNSDK_DOWNLOAD_STATE_ENUM, FUNSDK_MEDIA_PLAY_STATE_ENUM } from '../types/stream';
export declare enum SearcResultRecordEnum {
    RECORD_FILES = "recordfiles",
    RECORD_TIMELIST = "recordtimelist"
}
export type SearcResultRecordFile = {
    channel: number;
    filesize: number;
    filename: string;
    startDate: string;
    startTimeOfDay: string;
    startTimeOfYear: string;
    startTimeLong: number;
    endTimeOfDay: string;
    endTimeOfYear: string;
    endTimeLong: number;
    endTimeLong24Hours: number;
    alarmExFileInfo: string;
    fileTimeLong: number;
    streamType: number;
    startTimestamp: string;
};
export type RecordViewNativeProps = ViewProps & {
    onMediaPlayState: (event: {
        nativeEvent: {
            playSpeed: number;
            state: number;
        };
    }) => void;
    onShowRateAndTime: (event: {
        nativeEvent: {
            time: string;
            rate: string;
        };
    }) => void;
    onVideoBufferEnd: (event: {
        nativeEvent: {
            isBufferEnd: boolean;
        };
    }) => void;
    onSearchRecordByFilesResult: (event: {
        nativeEvent: {
            type: SearcResultRecordEnum;
            list: SearcResultRecordFile[];
        };
    }) => void;
    onFailed: (event: {
        nativeEvent: {
            msgId: number;
            errorId: number;
        };
    }) => void;
    onStartInit: (event: {
        nativeEvent: {};
    }) => void;
    onDebugState: (event: {
        nativeEvent: {
            state: string;
        };
    }) => void;
    onDownloadProgress?: (event: {
        nativeEvent: {
            progress: number;
        };
    }) => void;
    onDownloadState?: (event: {
        nativeEvent: {
            downloadState: FUNSDK_DOWNLOAD_STATE_ENUM;
            fileName: string;
        };
    }) => void;
};
export type RecordViewComponent = React.ComponentClass<RecordViewNativeProps>;
export declare const RecordView: import("react-native").HostComponent<RecordViewNativeProps>;
export type RecordPlayerProps = ViewProps & {
    devId: string;
    channelId: number;
    onMediaPlayState?: (obj: {
        playSpeed: number;
        state: FUNSDK_MEDIA_PLAY_STATE_ENUM;
    }) => void;
    onShowRateAndTime?: (obj: {
        time: string;
        rate: string;
    }) => void;
    onVideoBufferEnd?: (obj: {
        isBufferEnd: boolean;
    }) => void;
    onSearchRecordByFilesResult?: (obj?: {
        type: SearcResultRecordEnum;
        list: SearcResultRecordFile[];
    }) => void;
    onFailed?: (obj: {
        msgId: number;
        errorId: number;
    }) => void;
    onStartInit?: () => void;
    onDebugState?: (obj: {
        state: string;
    }) => void;
    onDownloadProgress?: (obj: {
        progress: number;
    }) => void;
    onDownloadState?: (obj: {
        downloadState: FUNSDK_DOWNLOAD_STATE_ENUM;
        fileName: string;
    }) => void;
};
export type RecordPlayerRef = {
    init: () => void;
    startPlayRecord: (position: number) => void;
    startPlayRecordByTime: (startTimestamp: number, endTimestamp: number) => void;
    searchRecordByFile: (startTimestamp: number, endTimestamp: number) => void;
    capture: (path: string) => void;
    startRecord: (path: string) => void;
    stopRecord: () => void;
    isRecording: () => void;
    openVoice: () => void;
    closeVoice: () => void;
    pausePlay: () => void;
    rePlay: () => void;
    stopPlay: () => void;
    destroyPlay: () => void;
    isRecordPlay: () => void;
    downloadFile: (position: number, path: string) => void;
    setPlaySpeed: (playSpeed: -3 | -2 | -1 | 0 | 1 | 2 | 3 | number) => void;
};
export declare const RecordPlayer: React.ForwardRefExoticComponent<ViewProps & {
    devId: string;
    channelId: number;
    onMediaPlayState?: ((obj: {
        playSpeed: number;
        state: FUNSDK_MEDIA_PLAY_STATE_ENUM;
    }) => void) | undefined;
    onShowRateAndTime?: ((obj: {
        time: string;
        rate: string;
    }) => void) | undefined;
    onVideoBufferEnd?: ((obj: {
        isBufferEnd: boolean;
    }) => void) | undefined;
    onSearchRecordByFilesResult?: ((obj?: {
        type: SearcResultRecordEnum;
        list: SearcResultRecordFile[];
    }) => void) | undefined;
    onFailed?: ((obj: {
        msgId: number;
        errorId: number;
    }) => void) | undefined;
    onStartInit?: (() => void) | undefined;
    onDebugState?: ((obj: {
        state: string;
    }) => void) | undefined;
    onDownloadProgress?: ((obj: {
        progress: number;
    }) => void) | undefined;
    onDownloadState?: ((obj: {
        downloadState: FUNSDK_DOWNLOAD_STATE_ENUM;
        fileName: string;
    }) => void) | undefined;
} & React.RefAttributes<RecordPlayerRef>>;
//# sourceMappingURL=index.d.ts.map